@@ -32,30 +32,16 @@ __all__ = [
 # System imports
 import sys
 
+from modelparameters.logger import debug, error, info
+
 # ModelParameters imports
-from modelparameters.sympytools import sp, Conditional
-from modelparameters.codegeneration import sympycode
+from modelparameters.sympytools import Conditional, sp
 
 # Local imports
-from gotran.common import (
-    error,
-    info,
-    debug,
-    check_arg,
-    check_kwarg,
-    scalars,
-    Timer,
-    warning,
-    tuplewrap,
-    parameters,
-    warning,
-)
-from gotran.common import listwrap
-from gotran.model.utils import ode_primitives
-from gotran.model.odeobjects import State, Parameter, IndexedObject, Comment
-from gotran.model.expressions import *
-from gotran.model.ode import ODE
-from gotran.codegeneration.codecomponent import CodeComponent
+from modelparameters.utils import Timer, check_arg
+
+from ..model.ode import ODE
+from .codecomponent import CodeComponent
 
 
 def explicit_euler_solver(ode, function_name="forward_explicit_euler", params=None):
@@ -78,10 +64,13 @@ def explicit_euler_solver(ode, function_name="forward_explicit_euler", params=No
 
 
 def rush_larsen_solver(
-    ode, function_name="forward_rush_larsen", delta=1e-8, params=None
+    ode,
+    function_name="forward_rush_larsen",
+    delta=1e-8,
+    params=None,
 ):
     """
-    Return an ODEComponent holding expressions for the Rush Larsen method
+    Return an ODEComponent holding expressions for the Rush-Larsen method
 
     Arguments
     ---------
@@ -90,7 +79,7 @@ def rush_larsen_solver(
     function_name : str
         The name of the function which should be generated
     delta : float
-        Value to safeguard the evaluation of the rush larsen step.
+        Value to safeguard the evaluation of the Rush-Larsen step.
     params : dict
         Parameters determining how the code should be generated
     """
@@ -101,10 +90,13 @@ def rush_larsen_solver(
 
 
 def generalized_rush_larsen_solver(
-    ode, function_name="forward_generalized_rush_larsen", delta=1e-8, params=None
+    ode,
+    function_name="forward_generalized_rush_larsen",
+    delta=1e-8,
+    params=None,
 ):
     """
-    Return an ODEComponent holding expressions for the generalized Rush Larsen method
+    Return an ODEComponent holding expressions for the generalized Rush-Larsen method
 
     Arguments
     ---------
@@ -113,7 +105,7 @@ def generalized_rush_larsen_solver(
     function_name : str
         The name of the function which should be generated
     delta : float
-        Value to safeguard the evaluation of the rush larsen step.
+        Value to safeguard the evaluation of the Rush-Larsen step.
     params : dict
         Parameters determining how the code should be generated
     """
@@ -121,7 +113,10 @@ def generalized_rush_larsen_solver(
         error("The ODE is not finalized")
 
     return GeneralizedRushLarsen(
-        ode, function_name=function_name, delta=delta, params=params
+        ode,
+        function_name=function_name,
+        delta=delta,
+        params=params,
     )
 
 
@@ -133,7 +128,7 @@ def hybrid_generalized_rush_larsen_solver(
     stiff_states=None,
 ):
     """
-    Return an ODEComponent holding expressions for the generalized Rush Larsen method
+    Return an ODEComponent holding expressions for the generalized Rush-Larsen method
 
     Arguments
     ---------
@@ -142,7 +137,7 @@ def hybrid_generalized_rush_larsen_solver(
     function_name : str
         The name of the function which should be generated
     delta : float
-        Value to safeguard the evaluation of the rush larsen step.
+        Value to safeguard the evaluation of the Rush-Larsen step.
     params : dict
         Parameters determining how the code should be generated
     stiff_state_variables : list of str
@@ -204,7 +199,7 @@ def get_solver_fn(solver_type):
 
 class ExplicitEuler(CodeComponent):
     """
-    An ODEComponent which compute one step of the explicit Euler algorithm
+    An ODEComponent which compute one step of the explicit Euler scheme
     """
 
     def __init__(self, ode, function_name="forward_explicit_euler", params=None):
@@ -227,8 +222,7 @@ class ExplicitEuler(CodeComponent):
 
         # Call base class using empty result_expressions
         descr = (
-            "Compute a forward step using the explicit Euler algorithm to the "
-            "{0} ODE".format(ode)
+            f"Compute a forward step using the explicit Euler scheme to the {ode} ODE"
         )
         super(ExplicitEuler, self).__init__(
             "ExplicitEuler",
@@ -247,12 +241,11 @@ class ExplicitEuler(CodeComponent):
         state_exprs = self.root.state_expressions
         states = self.root.full_states
         result_name = self._params.states.array_name
-        state_offset = self._params.states.add_offset
         self.shapes[result_name] = (len(states),)
 
         # Get time step and start creating the update algorithm
         if self._params.states.add_offset:
-            offset_str = "{0}_offset".format(result_name)
+            offset_str = f"{result_name}_offset"
         else:
             offset_str = ""
 
@@ -275,13 +268,53 @@ class ExplicitEuler(CodeComponent):
         self.body_expressions = self._recreate_body(body_expressions, **results)
 
 
+def fraction_numerator_is_nonzero(expr):
+    """Perform a very cheap check to detect if a fraction is definitely non-zero."""
+    expr_type = type(expr)
+    if expr_type == sp.Pow:
+        # check if the expression is on the form a**-1
+        a, b = expr.args
+        if type(b) is sp.numbers.NegativeOne:
+            return True
+        else:
+            # we won't do any further checks
+            return False
+    elif expr_type == sp.Mul:
+        # check if all factors are non-zero
+        args = expr.args
+        certainly_nonzero_args = []
+        potentially_nonzero_args = []
+        for e in args:
+            if len(e.free_symbols) == 0 and e.is_nonzero:
+                certainly_nonzero_args.append(e)
+            else:
+                potentially_nonzero_args.append(e)
+
+        if len(potentially_nonzero_args) == 0:
+            # all factors are certainly nonzero
+            return True
+
+        # check all potentially non-zero factors
+        for e in potentially_nonzero_args:
+            if not fraction_numerator_is_nonzero(e):
+                return False
+        else:
+            return True
+    else:
+        return False
+
+
 class RushLarsen(CodeComponent):
     """
-    An ODEComponent which compute one step of the Rush Larsen algorithm
+    An ODEComponent which compute one step of the Rush-Larsen scheme
     """
 
     def __init__(
-        self, ode, function_name="forward_rush_larsen", delta=1e-8, params=None
+        self,
+        ode,
+        function_name="forward_rush_larsen",
+        delta=1e-8,
+        params=None,
     ):
         """
         Create a RushLarsen Solver component
@@ -293,7 +326,7 @@ class RushLarsen(CodeComponent):
         function_name : str
             The name of the function which should be generated
         delta : float
-            Value to safeguard the evaluation of the rush larsen step.
+            Value to safeguard the evaluation of the Rush-Larsen step.
         params : dict
             Parameters determining how the code should be generated
         """
@@ -302,13 +335,10 @@ class RushLarsen(CodeComponent):
         check_arg(ode, ODE)
 
         if ode.is_dae:
-            error("Cannot generate a Rush Larsen forward step for a DAE.")
+            error("Cannot generate a Rush-Larsen forward step for a DAE.")
 
         # Call base class using empty result_expressions
-        descr = (
-            "Compute a forward step using the rush larsen algorithm to the "
-            "{0} ODE".format(ode)
-        )
+        descr = f"Compute a forward step using the Rush-Larsen scheme to the {ode} ODE"
         super(RushLarsen, self).__init__(
             "RushLarsen",
             ode,
@@ -327,23 +357,18 @@ class RushLarsen(CodeComponent):
         states = self.root.full_states
 
         result_name = self._params.states.array_name
-        state_offset = self._params.states.add_offset
         self.shapes[result_name] = (len(states),)
 
         # Get time step and start creating the update algorithm
         if self._params.states.add_offset:
-            offset_str = "{0}_offset".format(result_name)
+            offset_str = f"{result_name}_offset"
         else:
             offset_str = ""
 
         might_take_time = len(states) >= 10
 
         if might_take_time:
-            info(
-                "Calculating derivatives of {0}. Might take some time...".format(
-                    ode.name
-                )
-            )
+            info(f"Calculating derivatives of {ode.name}. Might take some time...")
             sys.stdout.flush()
 
         dt = self.root._dt.sym
@@ -359,20 +384,32 @@ class RushLarsen(CodeComponent):
             # print expr.state.sym, expr_diff, expr_diff.args
             if expr_diff and expr.state.sym not in expr_diff.args:
 
+                linearized_name = expr.name + "_linearized"
                 linearized = self.add_intermediate(
-                    expr.name + "_linearized", expr_diff, dependent=dependent
+                    linearized_name,
+                    expr_diff,
+                    dependent=dependent,
                 )
 
+                need_zero_div_check = not fraction_numerator_is_nonzero(expr_diff)
+                if not need_zero_div_check:
+                    debug(
+                        f"{linearized_name} cannot be zero. Skipping zero division check",
+                    )
+
+                RL_term = expr.sym / linearized * (sp.exp(linearized * dt) - 1)
+                if need_zero_div_check:
+                    RL_term = Conditional(
+                        abs(linearized) > delta,
+                        RL_term,
+                        dt * expr.sym,
+                    )
+
                 # Solve "exact" using exp
                 self.add_indexed_expression(
                     result_name,
                     (i,),
-                    expr.state.sym
-                    + Conditional(
-                        abs(linearized) > delta,
-                        expr.sym / linearized * (sp.exp(linearized * dt) - 1.0),
-                        dt * expr.sym,
-                    ),
+                    expr.state.sym + RL_term,
                     offset_str,
                     dependent=dependent,
                     enum=expr.state,
@@ -403,7 +440,7 @@ class RushLarsen(CodeComponent):
 
 class RushLarsenOneStep(CodeComponent):
     """
-    An ODEComponent which compute one step of the Rush Larsen algorithm
+    An ODEComponent which compute one step of the Rush-Larsen scheme
     """
 
     def __init__(self, ode, function_name="forward_rush_larsen", params=None):
@@ -422,13 +459,10 @@ class RushLarsenOneStep(CodeComponent):
         check_arg(ode, ODE)
 
         if ode.is_dae:
-            error("Cannot generate a Rush Larsen forward step for a DAE.")
+            error("Cannot generate a Rush-Larsen forward step for a DAE.")
 
         # Call base class using empty result_expressions
-        descr = (
-            "Compute a forward step using the rush larsen algorithm to the "
-            "{0} ODE".format(ode)
-        )
+        descr = f"Compute a forward step using the Rush-Larsen scheme to the {ode} ODE"
         state_name = params.states.array_name
         super(RushLarsen, self).__init__(
             "RushLarsen",
@@ -449,13 +483,12 @@ class RushLarsenOneStep(CodeComponent):
 
         result_name = self._params.states.array_name + "_1"
         previous_name = self._params.states.array_name + "_0"
-        state_offset = self._params.states.add_offset
         self.shapes[result_name] = (len(states),)
         self.shapes[previous_name] = (len(states),)
 
         # Get time step and start creating the update algorithm
         if self._params.states.add_offset:
-            offset_str = "{0}_offset".format(result_name)
+            offset_str = f"{result_name}_offset"
         else:
             offset_str = ""
 
@@ -472,14 +505,16 @@ class RushLarsenOneStep(CodeComponent):
             if expr_diff and expr.state.sym not in expr_diff:
 
                 linearized = self.add_intermediate(
-                    expr.name + "_linearized", expr_diff, dependent=dependent
+                    expr.name + "_linearized",
+                    expr_diff,
+                    dependent=dependent,
                 )
 
                 # Solve "exact" using exp
                 self.add_indexed_expression(
                     result_name,
                     (i,),
-                    prev + expr.sym / linearized * (sp.exp(linearized * dt) - 1.0),
+                    prev + expr.sym / linearized * (sp.exp(linearized * dt) - 1),
                     offset_str,
                     dependent=dependent,
                     enum=expr.state,
@@ -506,7 +541,7 @@ class RushLarsenOneStep(CodeComponent):
 
 class GeneralizedRushLarsen(CodeComponent):
     """
-    An ODEComponent which compute one step of the Generalized Rush Larsen (GRL1) algorithm
+    An ODEComponent which compute one step of the Generalized Rush-Larsen (GRL1) scheme
     """
 
     def __init__(
@@ -526,7 +561,7 @@ class GeneralizedRushLarsen(CodeComponent):
         function_name : str
             The name of the function which should be generated
         delta : float
-            Value to safeguard the evaluation of the rush larsen step.
+            Value to safeguard the evaluation of the Rush-Larsen step.
         params : dict
             Parameters determining how the code should be generated
         """
@@ -534,7 +569,7 @@ class GeneralizedRushLarsen(CodeComponent):
 
         # Call base class using empty result_expressions
         descr = (
-            "Compute a forward step using the rush larsen algorithm to the "
+            "Compute a forward step using the generalised Rush-Larsen (GRL1) scheme to the "
             "{0} ODE".format(ode)
         )
         super(GeneralizedRushLarsen, self).__init__(
@@ -554,13 +589,12 @@ class GeneralizedRushLarsen(CodeComponent):
         state_exprs = self.root.state_expressions
         states = self.root.full_states
         result_name = self._params.states.array_name
-        state_offset = self._params.states.add_offset
 
         self.shapes[result_name] = (len(states),)
 
         # Get time step and start creating the update algorithm
         if self._params.states.add_offset:
-            offset_str = "{0}_offset".format(result_name)
+            offset_str = f"{result_name}_offset"
         else:
             offset_str = ""
 
@@ -580,20 +614,30 @@ class GeneralizedRushLarsen(CodeComponent):
                 )
                 continue
 
+            linearized_name = expr.name + "_linearized"
             linearized = self.add_intermediate(
-                expr.name + "_linearized", expr_diff, dependent=dependent
+                linearized_name,
+                expr_diff,
+                dependent=dependent,
             )
 
+            need_zero_div_check = not fraction_numerator_is_nonzero(expr_diff)
+            if not need_zero_div_check:
+                debug(f"{linearized_name} cannot be zero. Skipping zero division check")
+
+            RL_term = expr.sym / linearized * (sp.exp(linearized * dt) - 1)
+            if need_zero_div_check:
+                RL_term = Conditional(
+                    abs(linearized) > delta,
+                    RL_term,
+                    dt * expr.sym,
+                )
+
             # Solve "exact" using exp
             self.add_indexed_expression(
                 result_name,
                 (i,),
-                expr.state.sym
-                + Conditional(
-                    abs(linearized) > delta,
-                    expr.sym / linearized * (sp.exp(linearized * dt) - 1.0),
-                    dt * expr.sym,
-                ),
+                expr.state.sym + RL_term,
                 offset_str,
                 dependent=dependent,
                 enum=expr.state,
@@ -608,7 +652,7 @@ class GeneralizedRushLarsen(CodeComponent):
 
 class HybridGeneralizedRushLarsen(CodeComponent):
     """
-    An ODEComponent which compute one step of the hybrid explicit Euler / Generalized Rush Larsen (GRL1) scheme
+    An ODEComponent which compute one step of the hybrid explicit Euler / Generalized Rush-Larsen (GRL1) scheme
     """
 
     def __init__(
@@ -629,7 +673,7 @@ class HybridGeneralizedRushLarsen(CodeComponent):
         function_name : str
             The name of the function which should be generated
         delta : float
-            Value to safeguard the evaluation of the rush larsen step.
+            Value to safeguard the evaluation of the Rush-Larsen step.
         params : dict
             Parameters determining how the code should be generated
 
@@ -637,10 +681,7 @@ class HybridGeneralizedRushLarsen(CodeComponent):
         check_arg(ode, ODE)
 
         # Call base class using empty result_expressions
-        descr = (
-            "Compute a forward step using the rush larsen algorithm to the "
-            "{0} ODE".format(ode)
-        )
+        descr = f"Compute a forward step using the FE / GRL1 scheme to the {ode} ODE"
         super(HybridGeneralizedRushLarsen, self).__init__(
             "HybridGeneralizedRushLarsen",
             ode,
@@ -659,7 +700,7 @@ class HybridGeneralizedRushLarsen(CodeComponent):
         for s in stiff_state_variables:
             if s == "":
                 continue
-            assert s in state_names, "Unknown state '{}'".format(s)
+            assert s in state_names, f"Unknown state '{s}'"
 
         # Recount the expressions if representation of states are "array" as
         # then the method is not full explcit
@@ -669,13 +710,12 @@ class HybridGeneralizedRushLarsen(CodeComponent):
         state_exprs = self.root.state_expressions
         states = self.root.full_states
         result_name = self._params.states.array_name
-        state_offset = self._params.states.add_offset
 
         self.shapes[result_name] = (len(states),)
 
         # Get time step and start creating the update algorithm
         if self._params.states.add_offset:
-            offset_str = "{0}_offset".format(result_name)
+            offset_str = f"{result_name}_offset"
         else:
             offset_str = ""
 
@@ -686,6 +726,7 @@ class HybridGeneralizedRushLarsen(CodeComponent):
             expr_diff = expr.expr.diff(expr.state.sym)
             dependent = expr if recount else None
             if not state_is_stiff or expr_diff.is_zero:
+                # FE scheme
                 self.add_indexed_expression(
                     result_name,
                     (i,),
@@ -696,20 +737,30 @@ class HybridGeneralizedRushLarsen(CodeComponent):
                 )
                 continue
 
+            linearized_name = expr.name + "_linearized"
             linearized = self.add_intermediate(
-                expr.name + "_linearized", expr_diff, dependent=dependent
+                linearized_name,
+                expr_diff,
+                dependent=dependent,
             )
 
+            need_zero_div_check = not fraction_numerator_is_nonzero(expr_diff)
+            if not need_zero_div_check:
+                debug(f"{linearized_name} cannot be zero. Skipping zero division check")
+
+            RL_term = expr.sym / linearized * (sp.exp(linearized * dt) - 1)
+            if need_zero_div_check:
+                RL_term = Conditional(
+                    abs(linearized) > delta,
+                    RL_term,
+                    dt * expr.sym,
+                )
+
             # Solve "exact" using exp
             self.add_indexed_expression(
                 result_name,
                 (i,),
-                expr.state.sym
-                + Conditional(
-                    abs(linearized) > delta,
-                    expr.sym / linearized * (sp.exp(linearized * dt) - 1.0),
-                    dt * expr.sym,
-                ),
+                expr.state.sym + RL_term,
                 offset_str,
                 dependent=dependent,
                 enum=expr.state,
@@ -725,7 +776,7 @@ class HybridGeneralizedRushLarsen(CodeComponent):
 class SimplifiedImplicitEuler(CodeComponent):
     """
     An ODEComponent which compute one step of a simplified Implicit Euler
-    algorithm
+    scheme
     """
 
     def __init__(
@@ -757,7 +808,7 @@ class SimplifiedImplicitEuler(CodeComponent):
         # Call base class using empty result_expressions
         descr = (
             "Compute a forward step using the simplified implicit Euler"
-            "algorithm to the {0} ODE".format(ode)
+            "scheme to the {0} ODE".format(ode)
         )
         super(SimplifiedImplicitEuler, self).__init__(
             "SimplifiedImplicitEuler",
@@ -777,12 +828,11 @@ class SimplifiedImplicitEuler(CodeComponent):
         states = self.root.full_states
 
         result_name = self._params.states.array_name
-        state_offset = self._params.states.add_offset
         self.shapes[result_name] = (len(states),)
 
         # Get time step and start creating the update algorithm
         if self._params.states.add_offset:
-            offset_str = "{0}_offset".format(result_name)
+            offset_str = f"{result_name}_offset"
         else:
             offset_str = ""
 
@@ -796,7 +846,9 @@ class SimplifiedImplicitEuler(CodeComponent):
 
             if not diag_jac_expr.is_zero:
                 diag_jac = self.add_intermediate(
-                    expr.name + "_diag_jac", diag_jac_expr, dependent=dependent
+                    expr.name + "_diag_jac",
+                    diag_jac_expr,
+                    dependent=dependent,
                 )
             else:
                 diag_jac = 0.0
