@@ -14,25 +14,30 @@
 #
 # You should have received a copy of the GNU Lesser General Public License
 # along with Gotran. If not, see <http://www.gnu.org/licenses/>.
-
 import re
-from future.standard_library import install_aliases
-
-install_aliases()
-import urllib.request, urllib.parse, urllib.error
-from xml.etree import ElementTree
+import urllib.error
+import urllib.parse
+import urllib.request
+from collections import defaultdict
+from collections import deque
+from collections import OrderedDict
 from functools import cmp_to_key
-
-from collections import OrderedDict, deque, defaultdict
-from gotran.common import info, warning, error, check_arg, begin_log, end_log
-from gotran.model.odeobjects import cmp
+from pathlib import Path
+from xml.etree import ElementTree
 
 from modelparameters.codegeneration import _all_keywords
-from modelparameters.parameterdict import *
+from modelparameters.logger import begin_log
+from modelparameters.logger import end_log
+from modelparameters.logger import error
+from modelparameters.logger import info
+from modelparameters.logger import warning
+from modelparameters.utils import check_arg
 
-# Local imports
 from gotran.common.options import parameters
 from gotran.input.mathml import MathMLBaseParser
+from gotran.model.odeobjects import cmp
+
+# Local imports
 
 __all__ = ["cellml2ode", "CellMLParser"]
 
@@ -128,7 +133,7 @@ class Equation(object):
         return self.name
 
     def __repr__(self):
-        return "Equation({0} = {1})".format(self.name, "".join(self.expr))
+        return f"Equation({self.name} = {''.join(self.expr)})"
 
     def __eq__(self, other):
         if not isinstance(other, Equation):
@@ -153,18 +158,18 @@ class Component(object):
             (state, variables.pop(state, None)) for state in state_variables
         )
 
-        for state, info in list(self.state_variables.items()):
-            self.variable_info[state] = info
+        for state, _info in list(self.state_variables.items()):
+            self.variable_info[state] = _info
             self.variable_info["type"] = "state_variable"
 
         self.parameters = OrderedDict(
-            (name, info)
-            for name, info in list(variables.items())
-            if info["init"] is not None
+            (name, _info)
+            for name, _info in list(variables.items())
+            if _info["init"] is not None
         )
 
-        for param, info in list(self.parameters.items()):
-            self.variable_info[param] = info
+        for param, _info in list(self.parameters.items()):
+            self.variable_info[param] = _info
             self.variable_info["type"] = "parameter"
 
         self.derivatives = state_variables
@@ -202,8 +207,7 @@ class Component(object):
 
         # Check if reserved name for state derivativeis is used as equation
         # name
-        derivative_names = ["d{0}_dt".format(der) for der in self.derivatives]
-        removal = []
+        derivative_names = [f"d{der}_dt" for der in self.derivatives]
 
         for eq in equations[:]:
             if (
@@ -240,10 +244,10 @@ class Component(object):
         return hash(self.name)
 
     def __str__(self):
-        return self.name + "<{0}>".format(len(self.state_variables))
+        return self.name + f"<{len(self.state_variables)}>"
 
     def __repr__(self):
-        return "Component<{0}, {1}>".format(self.name, len(self.state_variables))
+        return f"Component<{self.name}, {len(self.state_variables)}>"
 
     def __eq__(self, other):
         if not isinstance(other, Component):
@@ -289,7 +293,7 @@ class Component(object):
 
         warning(
             "Locally change parameter name '{0}' to '{1}' in "
-            "component '{2}'.".format(oldname, newname, self.name)
+            "component '{2}'.".format(oldname, newname, self.name),
         )
 
         # Update parameters
@@ -318,7 +322,7 @@ class Component(object):
 
         warning(
             "Locally change state name '{0}' to '{1}' in component "
-            "'{2}'.".format(oldname, newname, self.name)
+            "'{2}'.".format(oldname, newname, self.name),
         )
 
         # Update parameters
@@ -343,10 +347,11 @@ class Component(object):
                 eqn.name = newder
 
         # Update derivative equation
-        old_eq_name = "d{0}_dt".format(oldname)
-        new_eq_name = "d{0}_dt".format(newname)
+        old_eq_name = f"d{oldname}_dt"
+        new_eq_name = f"d{newname}_dt"
         self.variable_info[new_eq_name] = self.variable_info.pop(
-            old_eq_name, dict(init=None, unit="1", private=True, type="equation")
+            old_eq_name,
+            dict(init=None, unit="1", private=True, type="equation"),
         )
 
         return newname
@@ -364,7 +369,7 @@ class Component(object):
 
         warning(
             "Locally change equation name '{0}' to '{1}' in "
-            "component '{2}'.".format(oldname, newname, self.name)
+            "component '{2}'.".format(oldname, newname, self.name),
         )
 
         # Go through all equations and change the name used locally
@@ -386,7 +391,7 @@ class Component(object):
         if oldname not in self.variable_info:
             error(
                 "Cannot change variable name. {0} is not a variable "
-                "in component {1}".format(oldname, self.name)
+                "in component {1}".format(oldname, self.name),
             )
             return
 
@@ -427,7 +432,7 @@ class CellMLParser(object):
 
         targets = targets or []
         params = params or {}
-        check_arg(model_source, str)
+        check_arg(model_source, (str, Path))
         check_arg(targets, (list, dict))
         self._params = self.default_parameters()
         self._params.update(params)
@@ -440,12 +445,12 @@ class CellMLParser(object):
             try:
                 fp = urllib.request.urlopen(model_source)
                 self.cellml = ElementTree.parse(fp).getroot()
-            except:
+            except Exception:
                 error("ERROR: Unable to open " + model_source)
 
         self.model_source = model_source
         self.name = self.cellml.attrib["name"]
-        begin_log("Parsing CellML model: {0}".format(self.name))
+        begin_log(f"Parsing CellML model: {self.name}")
         self.mathmlparser = MathMLBaseParser(self._params.use_sympy_integers)
         self.cellml_namespace = self.cellml.tag.split("}")[0] + "}"
         self.parse_units()
@@ -481,7 +486,7 @@ class CellMLParser(object):
                 for par in child.iter(namespace + "para"):
                     # Get lines
                     splitted_line = deque(
-                        ("".join(text.strip() for text in par.itertext())).split(" ")
+                        ("".join(text.strip() for text in par.itertext())).split(" "),
                     )
 
                     # Cut them in lines which are not longer than 80 characters
@@ -511,7 +516,7 @@ class CellMLParser(object):
             content = ""
 
         if title or content:
-            return "%s\n\n%s" % (title, content)
+            return f"{title}\n\n{content}"
 
         return ""
 
@@ -546,13 +551,9 @@ class CellMLParser(object):
             collected_parts = OrderedDict()
             for unit in list(units):
                 if unit.attrib.get("multiplier"):
-                    warning(
-                        "skipping multiplier in unit {0}".format(units.attrib["name"])
-                    )
+                    warning(f"skipping multiplier in unit {units.attrib['name']}")
                 if unit.attrib.get("multiplier"):
-                    warning(
-                        "skipping multiplier in unit {0}".format(units.attrib["name"])
-                    )
+                    warning(f"skipping multiplier in unit {units.attrib['name']}")
                 cellml_unit = unit.attrib.get("units")
 
                 prefix = prefix_map[unit.attrib.get("prefix")]
@@ -568,9 +569,9 @@ class CellMLParser(object):
                     collected_parts[name] = (fullname, exponent)
                 elif cellml_unit in collected_units:
                     if prefix:
-                        warning("Skipping prefix of unit '{0}'".format(cellml_unit))
+                        warning(f"Skipping prefix of unit '{cellml_unit}'")
                     for name, (fullnam, part_exponent) in list(
-                        collected_units[cellml_unit].items()
+                        collected_units[cellml_unit].items(),
                     ):
                         new_exponent = float(part_exponent) * float(exponent)
 
@@ -591,7 +592,9 @@ class CellMLParser(object):
                     parsed_twice.append(units)
                     break
                 else:
-                    warning("Unknown unit '{0}' in ".format(cellml_unit, units["name"]))
+                    warning(
+                        "Unknown unit '{0}' in {1}".format(cellml_unit, units["name"]),
+                    )
 
             else:
                 # Try change mole*l**-1 to mM...
@@ -618,7 +621,11 @@ class CellMLParser(object):
         return item.iter(self.cellml_namespace + name)
 
     def check_and_register_component_variables(
-        self, comp, collected_states, collected_parameters, collected_equations
+        self,
+        comp,
+        collected_states,
+        collected_parameters,
+        collected_equations,
     ):
         """
         Check if component variables are allready collected
@@ -626,7 +633,7 @@ class CellMLParser(object):
 
         # Check for duplication of states
         for name in list(comp.state_variables.keys()):
-            der_name = "d{0}_dt".format(name)
+            der_name = f"d{name}_dt"
             if name in self._params.change_state_names:
                 newname = name + "_" + comp.name.split("_")[0]
                 comp.change_state_name(name, newname)
@@ -637,7 +644,7 @@ class CellMLParser(object):
                 state_comp = collected_states[name]
                 begin_log(
                     "Same state name: '{0}' is used in component: '{1}' "
-                    "and '{2}'.".format(name, comp.name, state_comp.name)
+                    "and '{2}'.".format(name, comp.name, state_comp.name),
                 )
                 for change_comp in [comp, state_comp]:
                     if (
@@ -652,7 +659,7 @@ class CellMLParser(object):
                     warning(
                         "Could not resolve duplicated state name {0} in "
                         "component {1} and {2}. None of them are private "
-                        "to the components.".format(name, comp.name, state_comp.name)
+                        "to the components.".format(name, comp.name, state_comp.name),
                     )
                 end_log()
 
@@ -662,8 +669,10 @@ class CellMLParser(object):
                 begin_log(
                     "State name: '{0}' from component '{1}' is used as "
                     "parameter in component '{2}'.".format(
-                        name, comp.name, param_comp.name
-                    )
+                        name,
+                        comp.name,
+                        param_comp.name,
+                    ),
                 )
 
                 # If parameter is private we change that
@@ -684,7 +693,7 @@ class CellMLParser(object):
                         "Could not resolve duplicated state and "
                         "parameter name {0} in component {1} and {2}. "
                         "None of them are private to the "
-                        "components.".format(name, comp.name, param_comp.name)
+                        "components.".format(name, comp.name, param_comp.name),
                     )
                 end_log()
 
@@ -694,8 +703,10 @@ class CellMLParser(object):
                 begin_log(
                     "State name '{0}' from component '{1}' is used as "
                     "parameter in component '{2}'.".format(
-                        name, comp.name, eq_comp.name
-                    )
+                        name,
+                        comp.name,
+                        eq_comp.name,
+                    ),
                 )
 
                 # If state is private we change it
@@ -706,15 +717,15 @@ class CellMLParser(object):
                     name = comp.change_state_name(name)
                 elif eq_comp.variable_info[name]["private"]:
                     new_name = eq_comp.change_equation_name(name)
-                    collected_equation.pop(name)
-                    collected_equation[new_name] = eq_comp
+                    collected_equations.pop(name)
+                    collected_equations[new_name] = eq_comp
 
                 else:
                     warning(
                         "Could not resolve duplicated state and "
                         "equation name {0} in component {1} and {2}. "
                         "None of them are private to the "
-                        "components.".format(name, comp.name, eq_comp.name)
+                        "components.".format(name, comp.name, eq_comp.name),
                     )
                 end_log()
 
@@ -727,12 +738,14 @@ class CellMLParser(object):
             # Check parameter name vs collected state names
             if name in collected_states:
                 state_comp = collected_states[name]
-                der_name = "d{0}_dt".format(name)
+                der_name = f"d{name}_dt"
                 begin_log(
                     "Same parameter and state name: '{0}' is used in "
                     "component '{1}' and '{2}'.".format(
-                        name, comp.name, state_comp.name
-                    )
+                        name,
+                        comp.name,
+                        state_comp.name,
+                    ),
                 )
                 # If parameter is private we change that
                 if comp.parameters[name]["private"]:
@@ -749,7 +762,7 @@ class CellMLParser(object):
                     warning(
                         "Could not resolve duplicated state name {0} in "
                         "component {1} and {2}. None of them are private "
-                        "to the components.".format(name, comp.name, state_comp.name)
+                        "to the components.".format(name, comp.name, state_comp.name),
                     )
                 end_log()
 
@@ -759,8 +772,10 @@ class CellMLParser(object):
                 begin_log(
                     "Parameter name '{0}' from component '{1}' is used as "
                     "parameter in component '{2}'.".format(
-                        name, comp.name, param_comp.name
-                    )
+                        name,
+                        comp.name,
+                        param_comp.name,
+                    ),
                 )
 
                 # If registered parameter is private we change that
@@ -778,7 +793,7 @@ class CellMLParser(object):
                         "Could not resolve duplicated parameter names "
                         "{0} in component {1} and {2}. "
                         "None of them are private to the "
-                        "components.".format(name, comp.name, param_comp.name)
+                        "components.".format(name, comp.name, param_comp.name),
                     )
                 end_log()
 
@@ -788,8 +803,10 @@ class CellMLParser(object):
                 begin_log(
                     "Parameter name '{0}' from component '{1}' "
                     "is used as parameter in component '{2}'.".format(
-                        name, comp.name, eq_comp.name
-                    )
+                        name,
+                        comp.name,
+                        eq_comp.name,
+                    ),
                 )
 
                 # If parameter is private we change it
@@ -797,15 +814,15 @@ class CellMLParser(object):
                     name = comp.change_parameter_name(name)
                 elif eq_comp.variable_info[name]["private"]:
                     new_name = eq_comp.change_equation_name(name)
-                    collected_equation.pop(name)
-                    collected_equation[new_name] = eq_comp
+                    collected_equations.pop(name)
+                    collected_equations[new_name] = eq_comp
 
                 else:
                     warning(
                         "Could not resolve duplicated parameter and "
                         "equation names {0} in component {1} and {2}. "
                         "None of them are private to the "
-                        "components.".format(name, comp.name, eq_comp.name)
+                        "components.".format(name, comp.name, eq_comp.name),
                     )
                 end_log()
 
@@ -829,12 +846,14 @@ class CellMLParser(object):
             # Check equation name vs collected state names
             if name in collected_states:
                 state_comp = collected_states[name]
-                der_name = "d{0}_dt".format(name)
+                der_name = f"d{name}_dt"
                 begin_log(
                     "Same equation and state name '{0}' is used in "
                     "component '{1}' and '{2}'.".format(
-                        name, comp.name, state_comp.name
-                    )
+                        name,
+                        comp.name,
+                        state_comp.name,
+                    ),
                 )
                 # If equation is private we change that
                 # if comp.variable_info[name]["private"]:
@@ -852,7 +871,7 @@ class CellMLParser(object):
                         "Could not resolve duplicated state and "
                         "equation name {0} in component {1} and {2}. "
                         "None of them are private to the "
-                        "components.".format(name, comp.name, state_comp.name)
+                        "components.".format(name, comp.name, state_comp.name),
                     )
                 end_log()
 
@@ -862,8 +881,10 @@ class CellMLParser(object):
                 begin_log(
                     "Equation name '{0}' from component '{1}' is used as "
                     "parameter in component '{2}'.".format(
-                        name, comp.name, param_comp.name
-                    )
+                        name,
+                        comp.name,
+                        param_comp.name,
+                    ),
                 )
 
                 # If registered parameter is private we change that
@@ -881,7 +902,7 @@ class CellMLParser(object):
                         "Could not resolve duplicated parameter and "
                         "equation name {0} in component {1} and {2}. "
                         "None of them are private to the "
-                        "components.".format(name, comp.name, param_comp.name)
+                        "components.".format(name, comp.name, param_comp.name),
                     )
 
                 end_log()
@@ -892,8 +913,10 @@ class CellMLParser(object):
                 info(
                     "Equation name '{0}' from component '{1}' is used as "
                     "equation name in component '{2}'.".format(
-                        name, comp.name, eq_comp.name
-                    )
+                        name,
+                        comp.name,
+                        eq_comp.name,
+                    ),
                 )
 
                 # If equation is private we change it
@@ -950,7 +973,10 @@ class CellMLParser(object):
         for comp in list(components.values()):
 
             self.check_and_register_component_variables(
-                comp, collected_states, collected_parameters, collected_equations
+                comp,
+                collected_states,
+                collected_parameters,
+                collected_equations,
             )
 
         return components, collected_states, collected_parameters, collected_equations
@@ -999,7 +1025,11 @@ class CellMLParser(object):
         return encapsulations, all_parents
 
     def parse_single_component(
-        self, comp, collected_states, collected_parameters, collected_equations
+        self,
+        comp,
+        collected_states,
+        collected_parameters,
+        collected_equations,
     ):
         """
         Parse a single component and create a Component object
@@ -1013,9 +1043,12 @@ class CellMLParser(object):
         # derivatives = []
 
         # Get variables that are used outside the component
-        variables_used_in_connections = list(
-            self.new_variable_connections.get(comp_name, dict()).keys()
-        ) + list(self.same_variable_connections.get(comp_name, dict()).keys())
+        variables_used_in_connections = (
+            list(
+                self.new_variable_connections.get(comp_name, dict()).keys(),
+            )
+            + list(self.same_variable_connections.get(comp_name, dict()).keys())
+        )
 
         # Get variable and initial values
         for var in self.get_iterator("variable", comp):
@@ -1043,9 +1076,12 @@ class CellMLParser(object):
         # Get equations
         for math in comp.iter("{http://www.w3.org/1998/Math/MathML}math"):
             for eq in list(math):
-                equation_list, state_variable, derivative, used_variables = self.mathmlparser.parse(
-                    eq
-                )
+                (
+                    equation_list,
+                    state_variable,
+                    derivative,
+                    used_variables,
+                ) = self.mathmlparser.parse(eq)
 
                 # Get equation name
                 eq_name = equation_list[0]
@@ -1057,7 +1093,7 @@ class CellMLParser(object):
                 # Discard collected equation name from used variables
                 used_variables.discard(eq_name)
 
-                assert re.findall("(\w+)", eq_name)[0] == eq_name
+                assert re.findall(r"(\w+)", eq_name)[0] == eq_name
                 assert equation_list[1] == self.mathmlparser["eq"]
                 equations.append(Equation(eq_name, equation_list[2:], used_variables))
 
@@ -1070,7 +1106,10 @@ class CellMLParser(object):
 
         # Collect and check variables
         self.check_and_register_component_variables(
-            comp, collected_states, collected_parameters, collected_equations
+            comp,
+            collected_states,
+            collected_parameters,
+            collected_equations,
         )
 
         return comp
@@ -1119,10 +1158,10 @@ class CellMLParser(object):
 
         try:
             import networkx as nx
-        except:
+        except ImportError:
             warning(
                 "networkx could not be imported. Circular "
-                "dependencies between components will not be sorted out."
+                "dependencies between components will not be sorted out.",
             )
             return sorted_components + circular_components
 
@@ -1156,7 +1195,7 @@ class CellMLParser(object):
                         equation_map[equation.name] = equation
                         one_dep_equations.add(equation)
                         one_dep_zero_dep[equation.name].add(
-                            equation.dependent_equations[0].name
+                            equation.dependent_equations[0].name,
                         )
 
         # Try to eliminate circular dependency
@@ -1239,7 +1278,7 @@ class CellMLParser(object):
                         sorted(
                             local_breakers,
                             key=cmp_to_key(lambda o0, o1: cmp(len(o0), len(o1))),
-                        )
+                        ),
                     ):
 
                         # If the removed edge is in the local edges
@@ -1297,7 +1336,7 @@ class CellMLParser(object):
                     new_dependent_componets[dep_comp] = old_comp.used_in.pop(dep_comp)
                 else:
                     new_dependent_componets[dep_comp] = [
-                        old_comp.used_in[dep_comp].pop(equations.index(eq))
+                        old_comp.used_in[dep_comp].pop(equations.index(eq)),
                     ]
 
                 # Change component dependencies
@@ -1330,9 +1369,7 @@ class CellMLParser(object):
         sort_again = False
         try:
             sorted_components = list(nx.topological_sort(G))
-            components.sort(
-                key = lambda n0: sorted_components.index(n0.name)
-            )
+            components.sort(key=lambda n0: sorted_components.index(n0.name))
             message = (
                 "To avoid circular dependency the following equations "
                 "has been moved:"
@@ -1352,11 +1389,7 @@ class CellMLParser(object):
         warning(message)
 
         for eq, old_comp in list(removed_equations.items()):
-            warning(
-                "{0} : from {1} to {2} component".format(
-                    eq.name, old_comp.name, ode_comp.name
-                )
-            )
+            warning(f"{eq.name} : from {old_comp.name} to {ode_comp.name} component")
 
         end_log()
 
@@ -1375,9 +1408,7 @@ class CellMLParser(object):
 
             try:
                 sorted_components = list(nx.topological_sort(G))
-                components.sort(
-                    key = lambda n0: sorted_components.index(n0.name)
-                )
+                components.sort(key=lambda n0: sorted_components.index(n0.name))
             except nx.NetworkXUnfeasible as e:
                 warning("Topological sort failed a second time: " + str(e))
 
@@ -1391,14 +1422,18 @@ class CellMLParser(object):
 
         # First parse connections which is used to determine the interface of
         # each variable
-        self.new_variable_connections, self.same_variable_connections = (
-            self.parse_connections()
-        )
+        (
+            self.new_variable_connections,
+            self.same_variable_connections,
+        ) = self.parse_connections()
 
         # Parse imported components
-        components, collected_states, collected_parameters, collected_equations = (
-            self.parse_imported_model()
-        )
+        (
+            components,
+            collected_states,
+            collected_parameters,
+            collected_equations,
+        ) = self.parse_imported_model()
 
         # Get parent relationship between components
         encapsulations, all_parents = self.get_parents(self._params.grouping)
@@ -1441,7 +1476,10 @@ class CellMLParser(object):
 
             # Store component
             components[comp_name] = self.parse_single_component(
-                comp, collected_states, collected_parameters, collected_equations
+                comp,
+                collected_states,
+                collected_parameters,
+                collected_equations,
             )
 
         # Add parent information
@@ -1542,7 +1580,8 @@ class CellMLParser(object):
                         if old_name_used:
                             for other_comp in old_name_used:
                                 components[other_comp].change_variable_name(
-                                    oldname, newname
+                                    oldname,
+                                    newname,
                                 )
 
                         for child in components[comp].children:
@@ -1662,48 +1701,50 @@ class CellMLParser(object):
                 names.appendleft(unders_score_replace(parent))
                 parent = parent.parent
 
-            comp_name = ", ".join('"{0}"'.format(name) for name in names)
+            comp_name = ", ".join(f'"{name}"' for name in names)
 
             # Collect initial state values
             if comp.state_variables:
                 declaration_lines.append("")
-                declaration_lines.append("states({0},".format(comp_name))
-                for name, info in list(comp.state_variables.items()):
-                    if info["unit"] != "1":
+                declaration_lines.append(f"states({comp_name},")
+                for name, _info in list(comp.state_variables.items()):
+                    if _info["unit"] != "1":
                         declaration_lines.append(
                             "       {0} = ScalarParam({1}"
                             ', unit="{2}"),'.format(
-                                name, float(info["init"]), info["unit"]
-                            )
+                                name,
+                                float(_info["init"]),
+                                _info["unit"],
+                            ),
                         )
                     else:
-                        declaration_lines.append(
-                            "       {0} = {1},".format(name, info["init"])
-                        )
+                        declaration_lines.append(f"       {name} = {_info['init']},")
                 declaration_lines[-1] = declaration_lines[-1][:-1] + ")"
 
             # Collect initial parameters values
             if comp.parameters:
                 declaration_lines.append("")
-                declaration_lines.append("parameters({0},".format(comp_name))
-                for name, info in list(comp.parameters.items()):
-                    if info["unit"] != "1":
+                declaration_lines.append(f"parameters({comp_name},")
+                for name, _info in list(comp.parameters.items()):
+                    if _info["unit"] != "1":
                         declaration_lines.append(
                             "           {0} = ScalarParam({1}"
                             ', unit="{2}"),'.format(
-                                name, float(info["init"]), info["unit"]
-                            )
+                                name,
+                                float(_info["init"]),
+                                _info["unit"],
+                            ),
                         )
                     else:
                         declaration_lines.append(
-                            "           {0} = {1},".format(name, info["init"])
+                            f"           {name} = {_info['init']},",
                         )
                 declaration_lines[-1] = declaration_lines[-1][:-1] + ")"
 
             # Collect all intermediate equations
             if comp.equations:
                 equation_lines.append("")
-                equation_lines.append("expressions({0})".format(comp_name))
+                equation_lines.append(f"expressions({comp_name})")
                 """
                 for eq in comp.equations:
                     expr = []
@@ -1728,8 +1769,7 @@ class CellMLParser(object):
                 )
 
         gotran_lines.append(
-            "# gotran file generated by cellml2gotran from "
-            "{0}".format(self.model_source)
+            f"# gotran file generated by cellml2gotran from {self.model_source}",
         )
         gotran_lines.extend(declaration_lines)
         gotran_lines.extend(equation_lines)
@@ -1740,7 +1780,7 @@ class CellMLParser(object):
         return "\n".join(gotran_lines)
 
         # Write file
-        open("{0}.ode".format(self.name), "w").write()
+        open(f"{self.name}.ode", "w").write()
 
 
 def cellml2ode(model_source, **options):
@@ -1754,7 +1794,7 @@ def cellml2ode(model_source, **options):
     options : dict
         Optional parameters to control cellml parser
     """
-    check_arg(model_source, str)
+    check_arg(model_source, (str, Path))
     from gotran.model.loadmodel import exec_ode
 
     params = CellMLParser.default_parameters()
